#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from PyQt5 import QtWidgets, uic
from pkg_resources import resource_filename
import logging
from ..helpers import loggingInfo


class DialogAddLocation(QtWidgets.QDialog):
    log = logging.getLogger('{}.DialogAddLocation'.format(__name__))

    @loggingInfo
    def __init__(self, parent):
        QtWidgets.QDialog.__init__(self, parent)
        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'dialogaddlocation.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)
        self.modal = True

    @loggingInfo
    def getData(self):
        name = str(self.ui.txtName.text())
        return name

    @loggingInfo
    def setData(self, name):
        self.ui.txtname.setText(name)


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)

    w = DialogAddLocation(None)
    w.show()
    sys.exit(app.exec_())


import inspect
from .gui.messagedialogs import MessageDialogs


def loggingInfo(f):
    def wrapper(obj, *args, **kwargs):
        try:
            f_name = f.__name__
            obj.log.info('Entering %s%s ', f_name, args)
            f_params = len((inspect.signature(f)).parameters)
            if f_params == 1:
                if len(args) > 0:
                    obj.log.info('Calling %s without params', f_name)
                result = f(obj)
            elif f_params >= 1:
                result = f(obj, *args, **kwargs)
            obj.log.info('Living %s ', f_name)
            return result
        except Exception as e:
            obj.log.error('Exception on %s: %s', f.__name__, e)
            raise e
    return wrapper


def catchError(msg=''):
    def wrapper(f):
        def _wrapper(obj, *args, **kwargs):
            try:
                return f(obj, *args, **kwargs)
            except Exception as e:
                if msg != '':
                    m = '{}. Icepap Error:\n{}'.format(msg, e)
                else:
                    m = 'Icepap Error:\n{}'.format(e)
                obj.log.error(m)
                MessageDialogs.showErrorMessage(None, 'Runtime Error', m)
        return _wrapper
    return wrapper


from PyQt5 import QtGui, uic
import re
from pkg_resources import resource_filename


class DialogStatusInfo(QtGui.QDialog):

    def __init__(self, parent, icepapsys, addr):
        QtGui.QDialog.__init__(self, parent)
        ui_filename = resource_filename('icepaposc.ui', 'dialogstatusinfo.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)

        self.icepapsys = icepapsys
        self.driver = icepapsys[addr]
        self.icepapAddress = icepapsys[addr].addr
        host = self.icepapsys._comm.host
        windows_name = 'Status Info  |  {}  |  {} {}'.format(host, addr,
                                                             self.driver.name)
        self.setWindowTitle(windows_name)
        self.show()
        self._connect_signals()
        self._do_vstatus()
        self.allDriversCommands = ['?cfg extdisable',
                                   '?ver info',
                                   '?power',
                                   '?positions',
                                   '?warning',
                                   '?alarm',
                                   '?isg ?ssierrtoggles',
                                   '#isg ssiwarningrst',
                                   'm ?ver info',
                                   'm ?rdispol',
                                   '?vstatus DISABLE',
                                   '?vstatus STOPCODE'
                                   ]
        for cmd in self.allDriversCommands:
            self.ui.cbAllDrivers.addItem(cmd)

        self.version_keys = [
            'CONTROLLER',
            'DSP',
            'FPGA',
            'MCPU1',
            'MCPU0',
            'MCPU2',
            'DRIVER']
        reg_exp = "({})\\s*:\\s*(\\d+\\.\\d+)".format(
            "|".join(self.version_keys))
        self.version_reg_exp = re.compile(reg_exp, re.VERBOSE)

    def _connect_signals(self):
        self.ui.btnUpdate.clicked.connect(self._do_vstatus)
        self.ui.btnUpdate.setDefault(False)
        self.ui.btnUpdate.setAutoDefault(False)
        self.ui.txt1Command.returnPressed.connect(self._send_command)
        self.ui.cbAllDrivers.activated.connect(self._send_command_to_drivers)

    def _do_vstatus(self):
        val = ""
        try:
            val = self.driver.vstatus
        except Exception as e:
            print(e)
        self.ui.textBrowser.setText(val)

    def _send_command(self):
        val = ""
        comm = ""
        try:

            comm = "" + str(self.ui.txt1Command.text())
            print(comm)
            val = self.driver.vstatus
            val = self.driver.send_cmd(comm)
            val = ' '.join(val)
            val = comm.upper() + " " + val
            print(val)
        except Exception as e:
            print(e)
        self.ui.textBrowser.setText(comm + "\n" + val)

    def _send_command_to_drivers(self):
        sel = self.ui.cbAllDrivers.currentText()
        sel_split = sel.split(' ')
        txt = ''
        if sel == '?positions':
            header = 'dr name axis absenc encin inpos tgtenc'
            txt = header + '\n'
            # for driver in self.driver.getDriversAlive():
            for driver in self.icepapsys.find_axes(only_alive=True):
                try:
                    val0 = self.icepapsys[driver].name
                    val1 = self.icepapsys[driver].pos
                    val2 = self.icepapsys[driver].enc_absenc
                    val3 = self.icepapsys[driver].enc_encin
                    val4 = self.icepapsys[driver].enc_inpos
                    val5 = self.icepapsys[driver].get_cfg('TGTENC')
                    val5 = val5['TGTENC']
                    if val0 == '':
                        val0 = 'noname'
                    txt = '{} {} {} {} {} {} {} {}\n'.format(txt, driver,
                                                             val0, val1, val2,
                                                             val3, val4, val5)
                    self.ui.textBrowser.setText(txt)
                except Exception as e:
                    print(e)
        elif sel == '?ver info':
            # for driver in self.driver.getDriversAlive():
            for driver in self.icepapsys.find_axes(only_alive=True):
                try:
                    val = self._get_version_info_dict(driver)
                    txt = '{} {} {}\n'.format(txt, driver, (val))
                    self.ui.textBrowser.setText(txt)
                except Exception as e:
                    print(e)
            self.ui.textBrowser.setText(txt)
        elif sel == 'm ?ver info':
            for contr in self._get_racks_alive():
                try:
                    val = self._get_version_info_dict(contr)
                    txt = '{} {} {}\n'.format(txt, contr, val)
                    self.ui.textBrowser.setText(txt)
                except Exception as e:
                    print(e)
            self.ui.textBrowser.setText(txt)
        elif sel.startswith('?vstatus') and len(sel_split) == 2:
            # for driver in self.driver.getDriversAlive():
            for driver in self.icepapsys.find_axes(only_alive=True):
                try:
                    comm = '{}:{}'.format(driver, '?vstatus')
                    val_lines = self.icepapsys.send_cmd(comm)
                    for ll in val_lines:
                        if sel_split[1] in ll:
                            txt = '{} {} {}\n'.format(txt, driver, ll)
                    self.ui.textBrowser.setText(txt)
                except Exception as e:
                    print(e)
            self.ui.textBrowser.setText(txt)
        elif sel.startswith('m '):
            print('master')
            contr_comm = sel.replace('m ', '')
            for contr in self._get_racks_alive():
                try:
                    comm = '{}:{}'.format(contr, contr_comm)
                    # val = self.driver.sendWriteReadCommand(comm)
                    val = self.icepapsys.send_cmd(comm)
                    val = ' '.join(val)
                    val = comm.upper() + " " + val
                    txt = '{} {} {}\n'.format(txt, contr, val)
                    self.ui.textBrowser.setText(txt)
                except Exception as e:
                    print(e)
            self.ui.textBrowser.setText(txt)
        else:
            for driver in self.icepapsys.find_axes(only_alive=True):
                try:
                    comm = '{}:{}'.format(driver,
                                          self.ui.cbAllDrivers.currentText())
                    val = self.icepapsys.send_cmd(comm)
                    val = ' '.join(val)
                    val = comm.upper() + " " + val
                    txt = '{} {} {}\n'.format(txt, driver, val)
                    self.ui.textBrowser.setText(txt)
                except Exception as e:
                    print(e)
            self.ui.textBrowser.setText(txt)

    def _get_version_info_dict(self, addr):
        command = "{}:?VER INFO".format(addr)
        ans = self.icepapsys.send_cmd(command)
        info = self.version_reg_exp.findall(str(ans))
        return dict(info)

    def _get_racks_alive(self):
        racks = []
        rackMask = 0
        try:
            rackMask = int(self.icepapsys.send_cmd('?sysstat')[0], 16)
        except BaseException:
            pass
        for rack in range(16):
            if (rackMask & (1 << rack)) != 0:
                racks.append(rack)
        return racks



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtGui, QtWidgets
import logging
from ..helpers import loggingInfo


class QValidateLineEdit(QtWidgets.QLineEdit):
    INTEGER, DOUBLE = list(range(2))
    log = logging.getLogger('{}.QValidateLineEdit'.format(__name__))

    @loggingInfo
    def __init__(self, parent, type, min, max):
        QtWidgets.QLineEdit.__init__(self, parent)
        self.type = type
        if type == QValidateLineEdit.INTEGER:
            self.myValidator = QtGui.QIntValidator(int(min),
                                                   int(max), self)
        elif type == QValidateLineEdit.DOUBLE:
            self.myValidator = QtGui.QDoubleValidator(float(min),
                                                      float(max), 2, self)
        else:
            self.myValidator = None

    @loggingInfo
    def keyPressEvent(self, event):
        QtWidgets.QLineEdit.keyPressEvent(self, event)
        self.setValidator(self.myValidator)
        if self.validator() != 0:
            palette = self.palette()
            if not self.hasAcceptableInput():
                palette.setColor(QtGui.QPalette.Text, QtGui.QColor('red'))
                self.setPalette(palette)
            else:
                palette.setColor(QtGui.QPalette.Text, QtGui.QColor('black'))
                self.setPalette(palette)
        self.setValidator(None)


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    m = QtWidgets.QWidget()
    w = QValidateLineEdit(m, QValidateLineEdit.INTEGER, -10, 10)
    m.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


import os
from PyQt5 import Qt, QtCore, QtWidgets
from xml.dom import minidom
import logging
from ..lib import ConfigManager
from ..helpers import loggingInfo

##########################################################################
# Templates Catalog Widget
# based on http://doc.trolltech.com/4.2/itemviews-basicsortfiltermodel.html
#
##########################################################################

CATALOG_PARAMS = ['MOTPHASES', 'MOTPOLES', 'MREGMODE', 'NVOLT', 'IVOLT',
                  'NCURR', 'CURRGAIN', 'MREGP', 'MREGI', 'MREGD', 'ANTURN',
                  'ANSTEP']

# 080731: jlidon considers that ICURR and BCURR are not needed
# 080731: jlidon considers that ANTURN adn ANSTEP must be part of the
#         catalog entries
# 090717: gcuni considers that catalogs should be param-free so any param
#         is possible


class TemplatesCatalogWidget(QtWidgets.QDialog):
    log = logging.getLogger('{}.TemplatesCatalogWidget'.format(__name__))

    @loggingInfo
    def __init__(self, master_catalog_file, pageipapdriver, parent=None):

        QtWidgets.QDialog.__init__(self, parent, QtCore.Qt.Window)

        self.pageipapdriver = pageipapdriver

        # BUILD THE CATALOG
        self.catalog = {}
        self.buildCatalog(master_catalog_file)
        config = ConfigManager()
        local_templates_dir = config.config[config.icepap]['templates_folder']
        prefix = local_templates_dir + os.sep
        local_templates = os.listdir(local_templates_dir)
        for local_template in local_templates:
            try:
                self.buildCatalog(prefix + local_template)
            except Exception:
                pass

        self.proxyModel = Qt.QSortFilterProxyModel()
        self.proxyModel.setDynamicSortFilter(True)
        self.proxyModel.setSourceModel(self.createModel())

        self.proxyGroupBox = QtWidgets.QGroupBox()
        self.proxyGroupBox.setFlat(True)

        self.proxyView = QtWidgets.QTreeView()
        self.proxyView.setRootIsDecorated(False)
        self.proxyView.setAlternatingRowColors(True)
        self.proxyView.setModel(self.proxyModel)
        self.proxyView.setSortingEnabled(True)
        # AT THE ISG LAB, THIS METHOD IS NOT AVAILABLE...
        self.proxyView.doubleClicked.connect(self.rowDoubleClicked)

        self.filterPatternLineEdit = QtWidgets.QLineEdit()
        self.filterPatternLabel = QtWidgets.QLabel("&Filter pattern:")
        self.filterPatternLabel.setBuddy(self.filterPatternLineEdit)
        self.filterPatternLineEdit.textChanged.connect(
            self.filterRegExpChanged)

        self.filterColumnComboBox = QtWidgets.QComboBox()
        # ITEMS HAVE BEEN FILLED AT THE self.buildCatalog CALL
        model = self.proxyModel.sourceModel()
        for column in range(model.columnCount()):
            self.filterColumnComboBox.addItem(
                model.horizontalHeaderItem(column).text())
        self.filterColumnLabel = QtWidgets.QLabel("Filter &column:")
        self.filterColumnLabel.setBuddy(self.filterColumnComboBox)
        self.filterColumnComboBox.currentIndexChanged.connect(
            self.filterColumnChanged)

        self.chkAutoClose = QtWidgets.QCheckBox("Auto close")
        self.chkAutoClose.setChecked(True)

        self.proxyLayout = QtWidgets.QGridLayout()

        self.proxyLayout.addWidget(self.filterColumnLabel, 0, 0)
        self.proxyLayout.addWidget(self.filterColumnComboBox, 0, 1)
        self.proxyLayout.addWidget(self.filterPatternLineEdit, 0, 2, 1, 2)
        self.proxyLayout.addWidget(self.chkAutoClose, 0, 4)

        self.proxyLayout.addWidget(self.proxyView, 1, 0, 1, 5)

        self.proxyGroupBox.setLayout(self.proxyLayout)

        self.mainLayout = QtWidgets.QVBoxLayout()
        self.mainLayout.addWidget(self.proxyGroupBox)
        self.setLayout(self.mainLayout)

        self.setWindowTitle("IcepaCMS Templates Catalog")
        self.resize(750, 600)

        self.proxyView.sortByColumn(0, QtCore.Qt.AscendingOrder)
        self.proxyView.setColumnWidth(0, 220)

        # BETTER FILTER ON DESCRIPTION BY DEFAULT
        self.filterColumnComboBox.setCurrentIndex(1)

        self.filterPatternLineEdit.setText("")

    @loggingInfo
    def rowDoubleClicked(self, modelindex):
        row = modelindex.row()
        # FIRST IT WAS DESIGNED TO NOT CLOSE
        # NOW, WE SHOULD RELY ON THE VALUE OF THE CHECKBOX
        # SO THE OPTION PARAMETER COULD BE REMOVED

        template_name = str(modelindex.sibling(row, 0).data())

        params = self.catalog.get(template_name)
        self.pageipapdriver.setTemplateParams(template_name, params)
        if self.chkAutoClose.isChecked():
            self.close()

    @loggingInfo
    def buildCatalog(self, catalog):
        doc = minidom.parse(catalog)
        root = doc.documentElement
        for template_node in root.getElementsByTagName('template'):
            template_name = template_node.attributes.get('template_name').value
            params = {}
            description = ""
            desc_node = template_node.getElementsByTagName('description')[
                0].firstChild
            if desc_node is not None:
                description = str(desc_node.data)
            params['description'] = description

            for param in list(template_node.attributes.keys()):
                param_value_instance = template_node.attributes.get(param)
                params[param] = param_value_instance.value
            self.catalog[str(template_name)] = params

    @loggingInfo
    def createModel(self, parent=None):
        model = Qt.QStandardItemModel(0, 2, self)
        model.setHeaderData(0, QtCore.Qt.Horizontal,
                            QtCore.QVariant("Template"))
        model.setHeaderData(1, QtCore.Qt.Horizontal,
                            QtCore.QVariant("Description"))

        for template_name in list(self.catalog.keys()):
            self.addTemplate(model, template_name,
                             self.catalog[template_name]['description'])

        return model

    @loggingInfo
    def addTemplate(self, model, template_name, description):
        model.insertRow(0)
        model.setData(model.index(0, 0), QtCore.QVariant(template_name))
        model.setData(model.index(0, 1), QtCore.QVariant(description))
        model.item(0, 0).setFlags(Qt.Qt.ItemIsSelectable)
        model.item(0, 1).setFlags(Qt.Qt.ItemIsSelectable)

    @loggingInfo
    def filterRegExpChanged(self):
        caseSensitivity = QtCore.Qt.CaseInsensitive
        syntax = QtCore.QRegExp.PatternSyntax(QtCore.QRegExp.RegExp)
        # Requested to use 'space' for joining words in an 'OR' operator
        filtertext = str(self.filterPatternLineEdit.text())
        filtertext = filtertext.lstrip().rstrip()
        words = filtertext.split(' ')
        if len(words) > 1:
            filtertext = '(' + ' | '.join(words) + ')'
        regExp = QtCore.QRegExp(str(filtertext), caseSensitivity, syntax)

        self.proxyModel.setFilterRegExp(regExp)
        # In case of just one match, auto-select it
        if self.proxyView.model().rowCount() == 1:
            self.proxyView.selectAll()
        else:
            self.proxyView.selectionModel().clear()

    @loggingInfo
    def filterColumnChanged(self):
        self.proxyModel.setFilterKeyColumn(
            self.filterColumnComboBox.currentIndex())

    @loggingInfo
    def closeEvent(self, event):
        self.close()
        event.accept()


if __name__ == '__main__':
    import sys
    from pkg_resources import resource_filename
    app = QtWidgets.QApplication(sys.argv)
    file_name = resource_filename('icepapcms.templates',
                                  'catalog.xml')
    w = TemplatesCatalogWidget(file_name, None)
    w.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtCore, QtGui, Qt, QtWidgets
import logging
from ..helpers import loggingInfo
from .icepapdriverwidget import IcePapDriverWidget


class PageiPapCrate(QtWidgets.QWidget):
    log = logging.getLogger('{}.PageiPapCrate'.format(__name__))

    @loggingInfo
    def __init__(self, mainwin):
        QtWidgets.QWidget.__init__(self, None)
        self.mainwin = mainwin
        self.vboxlayout = QtWidgets.QVBoxLayout(self)
        # TODO check if the margin is for all
        self.vboxlayout.setContentsMargins(9, 9, 9, 9)
        self.vboxlayout.setSpacing(6)

        self.tableWidget = QtWidgets.QTableWidget(self)
        palette = QtGui.QPalette()

        # Configure Palette Color Active
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(0),
            QtGui.QColor(16, 16, 16))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(1),
            QtGui.QColor(226, 228, 252))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(2),
            QtGui.QColor(237, 237, 237))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(3),
            QtGui.QColor(247, 245, 243))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(4),
            QtGui.QColor(119, 117, 115))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(5),
            QtGui.QColor(159, 157, 154))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(6),
            QtGui.QColor(16, 16, 16))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(7),
            QtGui.QColor(255, 255, 255))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(8),
            QtGui.QColor(16, 16, 16))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(9),
            QtGui.QColor(239, 235, 231))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(10),
            QtGui.QColor(239, 235, 231))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(11),
            QtGui.QColor(0, 0, 0))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(12),
            QtGui.QColor(101, 148, 235))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(13),
            QtGui.QColor(255, 255, 255))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(14),
            QtGui.QColor(0, 0, 255))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(15),
            QtGui.QColor(255, 0, 255))
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.ColorRole(16),
            QtGui.QColor(247, 245, 243))

        # Configure Palette Color Inactive
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(0),
            QtGui.QColor(16, 16, 16))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(1),
            QtGui.QColor(226, 228, 252))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(2),
            QtGui.QColor(237, 237, 237))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(3),
            QtGui.QColor(247, 245, 243))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(4),
            QtGui.QColor(119, 117, 115))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(5),
            QtGui.QColor(159, 157, 154))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(6),
            QtGui.QColor(16, 16, 16))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(7),
            QtGui.QColor(255, 255, 255))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(8),
            QtGui.QColor(16, 16, 16))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(9),
            QtGui.QColor(239, 235, 231))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(10),
            QtGui.QColor(239, 235, 231))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(11),
            QtGui.QColor(0, 0, 0))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(12),
            QtGui.QColor(101, 148, 235))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(13),
            QtGui.QColor(255, 255, 255))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(14),
            QtGui.QColor(0, 0, 255))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(15),
            QtGui.QColor(255, 0, 255))
        palette.setColor(QtGui.QPalette.Inactive, QtGui.QPalette.ColorRole(16),
            QtGui.QColor(247, 245, 243))

        # Configure Palette Color Disabled
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(0),
            QtGui.QColor(127, 125, 123))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(1),
            QtGui.QColor(226, 228, 252))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(2),
            QtGui.QColor(237, 237, 237))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(3),
            QtGui.QColor(247, 245, 243))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(4),
            QtGui.QColor(119, 117, 115))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(5),
            QtGui.QColor(159, 157, 154))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(6),
            QtGui.QColor(127, 125, 123))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(7),
            QtGui.QColor(255, 255, 255))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(8),
            QtGui.QColor(127, 125, 123))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(9),
            QtGui.QColor(255, 255, 255))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(10),
            QtGui.QColor(239, 235, 231))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(11),
            QtGui.QColor(0, 0, 0))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(12),
            QtGui.QColor(84, 123, 196))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(13),
            QtGui.QColor(255, 255, 255))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(14),
            QtGui.QColor(0, 0, 255))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(15),
            QtGui.QColor(255, 0, 255))
        palette.setColor(QtGui.QPalette.Disabled, QtGui.QPalette.ColorRole(16),
            QtGui.QColor(247, 245, 243))

        self.tableWidget.setPalette(palette)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setSelectionMode(
            QtWidgets.QAbstractItemView.NoSelection)
        self.tableWidget.setSelectionBehavior(
            QtWidgets.QAbstractItemView.SelectItems)
        self.tableWidget.setGridStyle(QtCore.Qt.SolidLine)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setMinimumSize(QtCore.QSize(40, 248))
        self.tableWidget.setMaximumSize(QtCore.QSize(112333, 248))

        spacerItem1 = QtWidgets.QSpacerItem(
            20, 40, QtWidgets.QSizePolicy.Minimum,
            QtWidgets.QSizePolicy.Expanding)
        self.vboxlayout.addItem(spacerItem1)
        self.vboxlayout.addWidget(self.tableWidget)
        spacerItem1 = QtWidgets.QSpacerItem(
            20, 40, QtWidgets.QSizePolicy.Minimum,
            QtWidgets.QSizePolicy.Expanding)
        self.vboxlayout.addItem(spacerItem1)

    @loggingInfo
    def fillData(self, icepap_system, selected_crate):
        self.tableWidget.clear()
        self.tableWidget.setColumnCount(8)
        self.tableWidget.setRowCount(0)
        self.tableWidget.horizontalHeader().setUpdatesEnabled(False)
        self.tableWidget.verticalHeader().setUpdatesEnabled(False)

        for i in range(8):
            headerItem = QtWidgets.QTableWidgetItem()
            headerItem.setText(str(i + 1))
            self.tableWidget.setHorizontalHeaderItem(i, headerItem)
            self.tableWidget.horizontalHeader().resizeSection(i, 94)
            self.tableWidget.horizontalHeader().setSectionResizeMode(
                i, Qt.QHeaderView.Custom)

        self.icepap_system = icepap_system
        self.cratenr = selected_crate
        self.driverswidgets = {}
        crate = -1
        row = 0
        """ TO-DO STORM review"""
        for driver in icepap_system.getDrivers():
            addr = driver.addr
            if driver.cratenr == selected_crate:
                crate = driver.cratenr
                self.tableWidget.insertRow(row)
                headerItem = QtWidgets.QTableWidgetItem()
                headerItem.setText("Crate %d" % crate)
                self.tableWidget.setVerticalHeaderItem(row, headerItem)
                self.tableWidget.verticalHeader().resizeSection(row, 200)
                self.tableWidget.verticalHeader().setSectionResizeMode(
                    row, Qt.QHeaderView.Custom)
                for drivernr in range(8):
                    addr = crate * 10 + drivernr + 1
                    adriver = icepap_system.getDriver(
                        crate * 10 + drivernr + 1)
                    if adriver is not None:
                        wdriver = IcePapDriverWidget(self)
                        wdriver.fillData(adriver)
                        self.driverswidgets[addr] = wdriver
                        self.tableWidget.setCellWidget(row, drivernr, wdriver)
                        wdriver.icepapDoubleClicked.connect(
                            self.driverDoubleclick)
                break

        self.tableWidget.horizontalHeader().setUpdatesEnabled(True)
        self.tableWidget.verticalHeader().setUpdatesEnabled(True)

    @loggingInfo
    def driverDoubleclick(self, driver):
        if driver is not None:
            location = "%s/%d/%d" % (driver.icepapsystem_name,
                                     driver.cratenr, driver.drivernr)
            self.mainwin.treeSelectByLocation(location)

    @loggingInfo
    def refresh(self):
        for col in range(self.tableWidget.columnCount()):
            for row in range(self.tableWidget.rowCount()):
                driver_widget = self.tableWidget.cellWidget(row, col)
                if driver_widget is not None:
                    if not driver_widget.refresh():
                        return


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    main_window = QtWidgets.QMainWindow()
    w = PageiPapCrate(main_window)
    w.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

__all__ = ['Conflict']


class Conflict:
    NO_CONFLICT, DRIVER_NOT_PRESENT, NEW_DRIVER, DRIVER_CHANGED, \
        NO_CONNECTION, DRIVER_CFG, DRIVER_MOVED, DRIVER_FROM_DB, \
        DRIVER_AUTOSOLVE, DRIVER_AUTOSOLVE_EXPERT = list(range(10))



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from .singleton import Singleton
import os
from configobj import ConfigObj
from validate import Validator
import logging
from ..helpers import loggingInfo

__all__ = ['ConfigManager']


class ConfigManager(Singleton):
    Sqlite = "sqlite"
    MySql = "mysql"
    Postgres = "postgres"
    database = "database"
    icepap = "icepap"
    folder = os.path.expanduser('~/.icepapcms/sqlitedb')
    log_folder = os.path.expanduser('~/.icepapcms/log')
    firmware_folder = os.path.expanduser('~/.icepapcms/firmware')
    configs_folder = os.path.expanduser('~/.icepapcms/configs')
    templates_folder = os.path.expanduser('~/.icepapcms/templates')
    snapshots_folder = os.path.expanduser('~/.icepapcms/snapshots')
    username = 'NotValidated'

    defaults = '''
    [database]
    password = string(default=configure)
    folder = string(default=''' + folder + ''')
    server = string(default=localhost:3306)
    user = string(default=icepapcms)
    database = string(default=sqlite)
    [icepap]
    debug_enabled = string(default=False)
    debug_level = string(default=1)
    log_folder = string(default=''' + log_folder + ''')
    firmware_folder = string(default=''' + firmware_folder + ''')
    configs_folder = string(default=''' + configs_folder + ''')
    templates_folder = string(default=''' + templates_folder + ''')
    snapshots_folder = string(default=''' + snapshots_folder + ''')
    [ldap]
    use = boolean(default=False)
    not_allowed = string(default='List of users no allowed to use the GUI')
    servers = string(default='list of servers')
    user_template=string(default='string with the configuration')
    [all_networks]
    use = boolean(default=False)
    
    
    '''

    defaults = defaults.splitlines()
    log = logging.getLogger('{}.ConfigManager'.format(__name__))

    def __init__(self):
        pass

    @loggingInfo
    def init(self, *args):
        if not os.path.exists(os.path.expanduser('~/.icepapcms')):
            os.mkdir(os.path.expanduser('~/.icepapcms'))
        self.config_filename = os.path.expanduser(
            '~/.icepapcms/icepapcms.conf')
        self.configure()

    @loggingInfo
    def configure(self):
        self.configspec = ConfigObj(self.defaults)
        self.config = ConfigObj(self.config_filename,
                                configspec=self.configspec)
        vdt = Validator()
        self.config.validate(vdt, copy=True)
        for folder in "log_folder", "firmware_folder", "configs_folder",\
                      "templates_folder", "snapshots_folder":
            directory = self.config["icepap"][folder]
            if not os.path.exists(directory):
                os.mkdir(directory)

    @loggingInfo
    def saveConfig(self):
        self.config.write()



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtCore, QtGui
from icepap import Mode
import logging
from ..helpers import loggingInfo
from ..lib import Conflict


class IcepapTreeModel(QtCore.QAbstractItemModel):
    SYSTEM, DRIVER, SYSTEM_WARNING, DRIVER_WARNING, SYSTEM_ERROR, \
        DRIVER_ERROR, CRATE, DRIVER_NEW, DRIVER_CFG, \
        SYSTEM_OFFLINE, DRIVER_MOVED, ROOT = list(range(12))

    log = logging.getLogger('{}.IcepapTreeModel'.format(__name__))

    @loggingInfo
    def __init__(self, IcepapsList, no_expand=False, parent=None):
        QtCore.QAbstractItemModel.__init__(self, parent)

        self.item_location = {}

        rootData = [QtCore.QVariant("IcepapCMS DB")]
        self.rootItem = TreeItem(rootData, IcepapTreeModel.ROOT, "DB")

        self.setupModelData(IcepapsList, self.rootItem, no_expand)
        self._dec_roles = (QtGui.QPixmap(":/icons/icons/ipapsys.png"),
                           QtGui.QPixmap(":/icons/icons/ipapdriver.png"),
                           QtGui.QPixmap(":/icons/icons/ipapsyswarning.png"),
                           QtGui.QPixmap(
                               ":/icons/icons/ipapdriverwarning.png"),
                           QtGui.QPixmap(":/icons/icons/ipapsyserror.png"),
                           QtGui.QPixmap(":/icons/icons/ipapdrivererror.png"),
                           QtGui.QPixmap(":/icons/icons/ipapcrate.png"),
                           QtGui.QPixmap(":/icons/icons/ipapdrivernew.png"),
                           QtGui.QPixmap(":/icons/icons/ipapdrivercfg.png"),
                           QtGui.QPixmap(":/icons/icons/ipapsysoffline.png"),
                           QtGui.QPixmap(":/icons/icons/ipapdrivermoved.png"),
                           QtGui.QPixmap(":/icons/icons/gnome-nettool.png"))

    @loggingInfo
    def columnCount(self, parent=QtCore.QModelIndex()):
        if parent.isValid():
            return parent.internalPointer().columnCount()
        else:
            return self.rootItem.columnCount()

    @loggingInfo
    def data(self, index, role):
        if not index.isValid():
            return QtCore.QVariant()
        if role == QtCore.Qt.DisplayRole:
            item = index.internalPointer()
            return QtCore.QVariant(item.data(index.column()))
        elif role == QtCore.Qt.DecorationRole:
            if (index.column() == 0):
                item = index.internalPointer()
                item.updateRole()
                return QtCore.QVariant(self._dec_roles[item.role])
            else:
                return QtCore.QVariant()

        elif role == QtCore.Qt.ToolTipRole:
            item = index.internalPointer()
            if item.description is None:
                return QtCore.QVariant()
            return QtCore.QVariant(item.description)
        elif role != QtCore.Qt.DisplayRole:
            return QtCore.QVariant()

    @loggingInfo
    def item_data(self, index):
        item = index.internalPointer()
        return item.itemData

    @loggingInfo
    def item(self, index):
        item = index.internalPointer()
        return item

    @loggingInfo
    def flags(self, index):
        if not index.isValid():
            return QtCore.Qt.ItemIsEnabled

        return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable

    @loggingInfo
    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
        if orientation == QtCore.Qt.Horizontal and \
                role == QtCore.Qt.DisplayRole:
            return self.rootItem.data(section)

        return QtCore.QVariant()

    @loggingInfo
    def index(self, row, column, parent=QtCore.QModelIndex()):
        if row < 0 or column < 0 or row >= self.rowCount(
                parent) or column >= self.columnCount(parent):
            return QtCore.QModelIndex()
        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()
        childItem = parentItem.child(row)
        if childItem:
            return self.createIndex(row, column, childItem)
        else:
            return QtCore.QModelIndex()

    @loggingInfo
    def parent(self, index):
        if not index.isValid():
            return QtCore.QModelIndex()
        childItem = index.internalPointer()
        parentItem = childItem.parent()
        if parentItem == self.rootItem:
            return QtCore.QModelIndex()
        return self.createIndex(parentItem.row(), 0, parentItem)

    @loggingInfo
    def rowCount(self, parent=QtCore.QModelIndex()):
        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()
        return parentItem.childCount()

    @loggingInfo
    def setupModelData(self, IcepapsList, parent, no_expand):
        keys = sorted(IcepapsList.keys())
        for icepap_name in keys:
            self.addIcepapSystem(
                icepap_name,
                IcepapsList.get(icepap_name),
                no_expand,
                parent)

    @loggingInfo
    def addIcepapSystem(self, icepap_name, icepap_system,
                        no_expand, parent=None, index=None):
        """ TO-DO STORM review"""
        if parent is None:
            parent = self.rootItem
        crate = -1
        location = icepap_name
        role = IcepapTreeModel.SYSTEM
        if no_expand:
            role = IcepapTreeModel.SYSTEM_OFFLINE

        new_item_system =\
            self.addItem(
                [QtCore.QVariant(icepap_name),
                 QtCore.QVariant(icepap_system.description)],
                role, location, icepap_system, parent, index)

        if icepap_system.conflict == Conflict.NO_CONNECTION or no_expand:
            return
        drivers = sorted(icepap_system.getDrivers(in_memory=True))
        for driver in drivers:
            addr = driver.addr
            if driver.cratenr != crate:
                crate = driver.cratenr
                location = "%s/%s" % (icepap_name, crate)
                new_item_crate = self.addItem([QtCore.QVariant(
                    driver.cratenr)], IcepapTreeModel.CRATE, location, None,
                    new_item_system)
            location = "%s/%s/%s" % (icepap_name, crate, addr)
            self.addItem([QtCore.QVariant(str(addr) + " " + driver.name)],
                         IcepapTreeModel.DRIVER, location, driver,
                         new_item_crate)

    @loggingInfo
    def deleteIcepapSystem(self, icepap_name):
        item = self.itemByLocation(icepap_name)
        self.deleteItem(item)
        return item

    @loggingInfo
    def updateIcepapSystem(self, icepap_system, no_expand=False):
        index = self.indexByLocation(icepap_system.name)
        item = self.deleteIcepapSystem(icepap_system.name)
        self.addIcepapSystem(
            icepap_system.name,
            icepap_system,
            no_expand,
            item.parent(),
            index)

    @loggingInfo
    def addItem(self, labels, role, location, data, parent, tree_index=None):
        new_item = TreeItem(labels, role, location, data, parent)
        parent.appendChild(new_item, tree_index)

        self.item_location[location] = new_item
        index = self.indexByLocation(location)
        self.beginInsertRows(self.parent(index), index.row(), index.row())
        self.endInsertRows()
        return new_item

    @loggingInfo
    def indexByLocation(self, location):
        if location in self.item_location:
            item = self.item_location[location]
            if item:
                return self.createIndex(item.row(), 0, item)
            else:
                None
        else:
            return None

    @loggingInfo
    def itemByLocation(self, location):
        if location in self.item_location:
            return self.item_location[location]
        else:
            return None

    @loggingInfo
    def deleteItem(self, item):
        index = self.indexByLocation(item.location)
        self.beginRemoveRows(self.parent(index), index.row(), index.row())
        del self.item_location[item.location]
        item.parentItem.removeChild(index.row())
        self.endRemoveRows()

    @loggingInfo
    def changeItemIcon(self, location, role):
        index = self.indexByLocation(location)
        if index is not None:
            modelitem = self.item(index)
            modelitem.role = role
            self.dataChanged.emit(index, index)


class TreeItem:
    log = logging.getLogger('{}.TreeItem'.format(__name__))

    @loggingInfo
    def __init__(self, label, role, location, data=None, parent=None):
        self.childItems = []
        self.description = None
        self.itemLabel = [label[0]]
        if len(label) > 1:
            if str(label[1].value()) != "":
                self.description = str(label[1].value)
        self.role = role
        self.itemData = data
        self.updateRole()
        self.parentItem = parent
        self.location = location

    @loggingInfo
    def updateRole(self):
        if self.role == IcepapTreeModel.SYSTEM:
            if self.itemData.conflict == Conflict.NO_CONNECTION:
                self.role = IcepapTreeModel.SYSTEM_ERROR
            elif self.itemData.child_conflicts > 0:
                self.role = IcepapTreeModel.SYSTEM_WARNING
        elif self.role == IcepapTreeModel.DRIVER or \
                self.role == IcepapTreeModel.DRIVER_CFG:
            if self.itemData.conflict == Conflict.NO_CONFLICT:
                self.role = IcepapTreeModel.DRIVER
            if self.itemData.conflict == Conflict.DRIVER_NOT_PRESENT:
                self.role = IcepapTreeModel.DRIVER_ERROR
            elif self.itemData.conflict == Conflict.NEW_DRIVER:
                self.role = IcepapTreeModel.DRIVER_NEW
            elif self.itemData.conflict == Conflict.DRIVER_CHANGED:
                self.role = IcepapTreeModel.DRIVER_WARNING
            elif self.itemData.conflict == Conflict.DRIVER_MOVED:
                self.role = IcepapTreeModel.DRIVER_MOVED
            elif self.itemData.mode == Mode.CONFIG:
                self.role = IcepapTreeModel.DRIVER_CFG
        return self.role

    @loggingInfo
    def appendChild(self, child, index=None):
        if index is None:
            self.childItems.append(child)
        else:
            self.childItems.insert(index.row(), child)

    @loggingInfo
    def removeChild(self, row):
        del self.childItems[row]

    @loggingInfo
    def child(self, row):
        return self.childItems[row]

    @loggingInfo
    def childCount(self):
        return len(self.childItems)

    @loggingInfo
    def columnCount(self):
        return len(self.itemLabel)

    @loggingInfo
    def data(self, column):
        return self.itemLabel[column]

    @loggingInfo
    def parent(self):
        return self.parentItem

    @loggingInfo
    def changeLabel(self, label):
        if len(label) > 1:
            if label[1] != "":
                self.description = label[1]

        self.itemLabel = [label[0]]

    @loggingInfo
    def row(self):
        if self.parentItem:
            return self.parentItem.childItems.index(self)
        return 0

    @loggingInfo
    def solveConflict(self):
        if self.role == IcepapTreeModel.DRIVER_WARNING or \
                self.role == IcepapTreeModel.DRIVER_NEW:
            self.role = IcepapTreeModel.DRIVER
            self.itemData.conflict = Conflict.NO_CONFLICT
            self.parentItem.notifySolvedConflict()
        elif self.role == IcepapTreeModel.DRIVER_ERROR:
            self.parentItem.notifySolvedConflict()

    @loggingInfo
    def notifySolvedConflict(self):
        if self.role == IcepapTreeModel.SYSTEM_WARNING:
            self.itemData.child_conflicts -= 1
            if self.itemData.child_conflicts == 0:
                self.role = IcepapTreeModel.SYSTEM
        elif self.role == IcepapTreeModel.CRATE:
            self.parentItem.notifySolvedConflict()

    @loggingInfo
    def getIcepapSystem(self):
        if self.role == IcepapTreeModel.SYSTEM or \
                self.role == IcepapTreeModel.SYSTEM_WARNING or \
                self.role == IcepapTreeModel.SYSTEM_ERROR or \
                self.role == IcepapTreeModel.SYSTEM_OFFLINE:
            return self.itemData
        if self.role == IcepapTreeModel.CRATE:
            return self.parentItem.getIcepapSystem()
        else:
            return self.parentItem.getIcepapSystem()



#!/usr/bin/env python

# ------------------------------------------------------------------------------
# This file is part of icepapCMS (https://github.com/ALBA-Synchrotron/icepapcms)
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# ------------------------------------------------------------------------------

from PyQt5.QtWidgets import QDialog, QDialogButtonBox
from PyQt5.QtCore import QTimer
from PyQt5 import uic
from pkg_resources import resource_filename
from .messagedialogs import MessageDialogs


class DialogHomeSrch(QDialog):

    def __init__(self, parent, axis):
        QDialog.__init__(self, parent)
        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'dialoghomesrch.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)
        self.parent = parent
        self.axis = axis
        self.close_button = self.ui.bbClose.button(QDialogButtonBox.Close)
        self.motor_moving = False
        self._set_go_stop_btn_layout()
        self._cb_hs_changed()
        self.tick_interval = 200  # [milliseconds]
        self.ticker = QTimer()
        self._connect_signals()

    def _connect_signals(self):
        self.ui.cbHomeSearch.currentIndexChanged.connect(self._cb_hs_changed)
        self.ui.cbHsOptions.currentIndexChanged.connect(self._update_access)
        self.ui.btnGoStop.clicked.connect(self._btn_go_stop_clicked)
        self.close_button.clicked.connect(self.close)
        self.ticker.timeout.connect(self._tick_status)

    def _update_access(self):
        have_home = self.ui.cbHomeSearch.currentText() == 'HOME'
        have_limit = self.ui.cbHsOptions.currentText() in ['Lim-', 'Lim+']
        disable = have_home or have_limit
        self.ui.cbEdge.setDisabled(disable)
        self.ui.cbDirection.setDisabled(disable)

    def _cb_hs_changed(self):
        home_selected = self.ui.cbHomeSearch.currentText() == 'HOME'
        if home_selected:
            items = ['-1', '0', '+1']
        else:
            items = ['Lim-', 'Lim+', 'Home', 'EncAux', 'InpAux']
        self.ui.cbHsOptions.clear()
        self.ui.cbHsOptions.addItems(items)
        self._update_access()

    def _set_go_stop_btn_layout(self):
        if self.motor_moving:
            self.ui.btnGoStop.setStyleSheet("background-color: red")
            self.ui.btnGoStop.setText('STOP')
        else:
            self.ui.btnGoStop.setStyleSheet("background-color: green")
            self.ui.btnGoStop.setText('GO')

    def _btn_go_stop_clicked(self):
        if self.motor_moving:
            self.motor_moving = False
            self.axis.stop()
        else:
            self.motor_moving = True
            self._start_motor()
        self._set_go_stop_btn_layout()

    def _start_motor(self):
        cmd = self.ui.cbHomeSearch.currentText()
        opt = self.ui.cbHsOptions.currentText()
        try:
            if cmd == 'HOME':
                direction = int(opt)
                self.axis.home(direction)
            else:
                edge = self.ui.cbEdge.currentText()
                direction = self.ui.cbDirection.currentText()
                self.axis.srch(opt, edge, direction)
        except RuntimeError as e:
            msg = 'HOME/SRCH failed:\n{}'.format(e)
            print(msg)
            MessageDialogs.showErrorMessage(None, 'HOME/SRCH', msg)
            return
        self.ticker.start(self.tick_interval)

    def _tick_status(self):
        try:
            if self.ui.cbHomeSearch.currentText() == 'HOME':
                status, direction = self.axis.homestat
                f1 = self.axis.get_home_position
                f2 = self.axis.get_home_encoder
            else:
                status, direction = self.axis.srchstat
                f1 = self.axis.get_srch_position
                f2 = self.axis.get_srch_encoder
            if status == 'MOVING':
                ss = "background-color: yellow"
                self.ui.gvIndicator.setStyleSheet(ss)
                if direction == -1:
                    pass
                elif direction == 1:
                    pass
                else:
                    self.axis.stop()
                    msg = 'Internal Error: Bad direction'
                    print(msg)
                    MessageDialogs.showErrorMessage(None, 'HOME/SRCH', msg)
                    return
                self.ticker.start(self.tick_interval)
            elif status == 'NOTFOUND':
                ss = "background-color: red"
                self.ui.gvIndicator.setStyleSheet(ss)
                self.motor_moving = False
                self._set_go_stop_btn_layout()
            else:
                ss = "background-color: green"
                self.ui.gvIndicator.setStyleSheet(ss)
                self.motor_moving = False
                self._set_go_stop_btn_layout()
                self.ui.lcdPosAxis.display(f1('AXIS'))
                self.ui.lcdPosTgtenc.display(f1('INPOS'))
                self.ui.lcdPosEncin.display(f1('TGTENC'))
                self.ui.lcdPosInpos.display(f1('ENCIN'))
                self.ui.lcdPosAbsenc.display(f1('ABSENC'))
                self.ui.lcdEncAxis.display(f2('AXIS'))
                self.ui.lcdEncTgtenc.display(f2('INPOS'))
                self.ui.lcdEncEncin.display(f2('TGTENC'))
                self.ui.lcdEncInpos.display(f2('ENCIN'))
                self.ui.lcdEncAbsenc.display(f2('ABSENC'))
        except RuntimeError as e:
            msg = 'HOME/SRCH failed:\n{}'.format(e)
            print(msg)
            MessageDialogs.showErrorMessage(None, 'HOME/SRCH', msg)

    def closeEvent(self, event):
        self.parent.enable_home_srch_button()
        event.accept()



# !/usr/bin/env python
# ------------------------------------------------------------------------------
# This file is part of IcepapCMS
#      https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# ------------------------------------------------------------------------------

#Remove the QT depency on the update driver
from PyQt5 import Qt
from icepap import IcePAPController, Mode
from pkg_resources import resource_filename
from xml.dom import minidom, Node
import os
import re
import socket
import collections
from IPy import IP
import datetime
import logging
import time
from .singleton import Singleton
from .icepapdrivercfg import IcepapDriverCfg
from . import icepapdriver
from .configmanager import ConfigManager
from ..gui.messagedialogs import MessageDialogs
from ..helpers import loggingInfo, catchError

__all__ = ['IcepapsManager']


class IcepapsManager(Singleton):
    log = logging.getLogger('{}.IcepapController'.format(__name__))

    def __init__(self):
        pass

    @loggingInfo
    def init(self, *args):
        self.iPaps = {}
        self.config_template = resource_filename('icepapcms.templates',
                                                 'driverparameters.xml')
        self._parseDriverTemplateFile()
        self._config = ConfigManager()
        self.icepap_cfginfos = {}
        self.programming_ipap = None
        try:
            ipap = self._config.icepap
            debug_set = self._config.config[ipap]["debug_enabled"]
            self.debug = debug_set == str(True)
            self.log_folder = self._config.config[ipap]["log_folder"]
            if not os.path.exists(self.log_folder):
                os.mkdir(self.log_folder)
        except Exception as e:
            self.log.error("Init error %s", e)
            pass

    @loggingInfo
    def reset(self):
        self.closeAllConnections()
        self.iPaps = {}

    @loggingInfo
    def openConnection(self, icepap_name, host, port):
        if not self.host_in_same_subnet(icepap_name):
            MessageDialogs.showInformationMessage(None, "Host connection",
                                                  "It is not allowed to "
                                                  "connect to %s. (Check "
                                                  "subnet)" % host)
            return False
        else:
            try:
                # TODO Optimize GUI to avoid auto_axes=True
                self.iPaps[icepap_name] = IcePAPController(host, int(port),
                                                           auto_axes=True)
                return True
            except Exception as e:
                self.log.error('Open Connection error %s', e)
                return False

    @loggingInfo
    def closeConnection(self, icepap_name):
        if icepap_name in self.iPaps:
            self.iPaps[icepap_name].disconnect()
            del self.iPaps[icepap_name]

    @loggingInfo
    def closeAllConnections(self):
        for iPap in list(self.iPaps.values()):
            try:
                iPap.disconnect()
            except Exception:
                pass
        self.iPaps = {}

    @loggingInfo
    def _get_driver_cfg_info(self, icepap_name, driver_addr):
        try:
            cfg_info = self.iPaps[icepap_name][driver_addr].get_cfginfo()
        except RuntimeError as e:
            msg = 'Failed to retrieve cfginfo ' \
                  '({0}).\n{1}'.format(driver_addr, e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'Driver Config Info', msg)
            raise e
        for key, val in list(cfg_info.items()):
            if val.startswith('{'):
                val = val.replace("{", "")
                val = val.replace("}", "")
            val = val.split()
            cfg_info[key] = val
        return cfg_info

    @loggingInfo
    def scanIcepapSystem(self, icepap_name, compare=False):
        """
        Get the status of the icepap system, the drivers present, and its
        configuration.
        """
        driver_name = icepap_name
        if compare:
            driver_name = "compare"

        driver_list = {}
        self.icepap_cfginfos[icepap_name] = collections.OrderedDict()

        try:
            # IMPROVE OF SPEED BY SAVING CFGINFOS BY VERSION
            cfginfos_version_dict = {}
            for addr in self.iPaps[icepap_name]:
                """ TO-DO STORM review"""
                driver = icepapdriver.IcepapDriver(driver_name, addr)
                driver_cfg = self.getDriverConfiguration(icepap_name, addr)
                driver.addConfiguration(driver_cfg)

                # CFGINFO IS ALSO SPECIFIC FOR EACH DRIVER
                # To improve speed, this is true but instead of 'each driver',
                # better each version
                driver_version = driver_cfg.getParameter('VER', True)
                if driver_version not in cfginfos_version_dict:
                    cfginfos_version_dict[driver_version] = \
                        self._get_driver_cfg_info(icepap_name, addr)

                # GET CFGINFO FROM CACHED DICT INSTEAD OF QUERYING EACH TIME
                cfginfo_dict = cfginfos_version_dict[driver_version]
                self.icepap_cfginfos[icepap_name][addr] = cfginfo_dict

                driver.setName(driver_cfg.getParameter("IPAPNAME", True))
                driver.setMode(self.iPaps[icepap_name][addr].mode)
                driver_list[addr] = driver

        except Exception as e:
            self.closeConnection(icepap_name)
            self.log.error('Scan Icepap System error: %s', e)
            raise
        return driver_list

    @loggingInfo
    def getDriverConfiguration(self, icepap_name, driver_addr):
        """
            Returns a IcepaDriverCfg object of the attributes predefined in
            driverparameters.xml
        """

        """ TO-DO STORM review"""
        driver_cfg = IcepapDriverCfg(str(datetime.datetime.now()))
        axis = self.iPaps[icepap_name][driver_addr]
        try:
            driver_cfg.setSignature(axis.config)
        except Exception as e:
            msg = 'Failed to retrieve configuration for ' \
                  'driver {0}.\n{1}'.format(driver_addr, e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'Get Driver Config', msg)
            raise e

        # THE VERSION NUMBER TO BE SHOWN IS THE DRIVER'S VERSION INSTEAD OF
        # THE DSP'S ONE.
        axis_name = axis.name
        # FIX NON-ASCII CHARS ISSUE IN NAME:
        if axis_name is not None and not all(ord(c) < 128 for c in axis_name):
            axis_name = repr(axis.name)

        driver_cfg.setParameter("VER", axis.fver)
        driver_cfg.setParameter("ID", axis.id)
        try:
            driver_cfg.setParameter("IPAPNAME", axis_name)
        except Exception as e:
            self.log.error('Exception when trying to write the driver name ' \
                           '(%s): %s', axis, e)

            axis_name = 'NON-ASCII_NAME'
            driver_cfg.setParameter("IPAPNAME", axis_name)

        # INSTEAD OF READING PARAM BY PARAM, WE SHOULD ASK THE ICEPAP FOR
        # ALL THE CONFIGURATION
        # WITH THE #N?:CFG COMMAND, USING SOME .getCfg() METHOD.
        axis = self.iPaps[icepap_name][driver_addr]
        cfg = axis.get_cfg()
        for param_name, param_value in list(cfg.items()):
            driver_cfg.setParameter(str(param_name), param_value)

        return driver_cfg

    @loggingInfo
    def setDriverConfiguration(self, icepap_name, driver_addr, new_values,
                               expertFlag=False):
        """ TO-DO STORM review"""
        axis = self.iPaps[icepap_name][driver_addr]
        try:
            if Mode.CONFIG != axis.mode:
                axis.set_config()
        except RuntimeError as e:
            msg = 'Failed to set driver {0} in CONFIG mode.\n{1}'.format(
                driver_addr, e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'Set Driver Mode', msg)
            raise e

        # Convert the new_values list in a dictionary
        new_values_dict = {}
        for name, value in new_values:
            new_values_dict[name] = value

        # THE CONFIGURATION VALUES SHOULD BE SENT IN A SPECIFIC ORDER
        cfg_info = self.icepap_cfginfos[icepap_name][driver_addr]
        for cfg_name in cfg_info:
            if cfg_name in new_values_dict:
                try:
                    value = str(new_values_dict.pop(cfg_name))
                    axis.set_cfg(cfg_name, value)
                except Exception as e:
                    msg = 'Error configuring parameter {0} = {1} for driver ' \
                          '{2}\n{3}'.format(cfg_name, value, driver_addr, e)
                    self.log.error(msg)
                    MessageDialogs.showErrorMessage(None, 'Set Driver Config',
                                                    msg)
                    raise e

        # The new_values will have the the not found parameter on the
        # configuration dictionary
        for cfg_name, value in list(new_values_dict.items()):
            if cfg_name.upper() in ['VER', 'ID']:
                # Ignore the change
                pass
            elif cfg_name.upper() in ['NAME', 'IPAPNAME']:
                have_namelock = False
                try:
                    cfg = axis.get_cfg()
                    # IN CASE OF NAMELOCK SET TO YES, UNLOCK FIRST
                    have_namelock = cfg['NAMELOCK'].upper() == 'YES'
                    if have_namelock:
                        axis.set_cfg('NAMELOCK', 'NO')
                    axis.name = value
                except Exception as e:
                    msg = 'Error configuring parameter ' \
                          '{0} = {1}\n{2}'.format(cfg_name, value, e)
                    self.log.error(msg)
                    MessageDialogs.showErrorMessage(None, 'Set Driver Config',
                                                    msg)
                finally:
                    if have_namelock:
                        axis.set_cfg('NAMELOCK', 'YES')
            else:
                msg = 'Parameter {0}({1}) is not exist for the current ' \
                      'version. Check the configuration ' \
                      'parameter list ({2}).'.format(cfg_name, value,
                                                     list(cfg_info.keys()))
                self.log.error(msg)
                MessageDialogs.showErrorMessage(None, 'Set Driver Config', msg)
                raise ValueError(msg)
        try:
            if expertFlag:
                axis.set_cfg('EXPERT')
        except Exception as e:
            msg = 'Error setting expert flag.{0}\n'.format(e)
            MessageDialogs.showErrorMessage(None, 'Expert flag', msg)
            self.log.error(msg)
            raise e

        driver_cfg = self.getDriverConfiguration(icepap_name, driver_addr)
        return driver_cfg

    @loggingInfo
    def discardDriverCfg(self, icepap_name, driver_addr):
        try:
            if icepap_name in self.iPaps:
                self.iPaps[icepap_name][driver_addr].set_config()
        except RuntimeError as e:
            msg = 'Error discarding driver config.\n{0}'.format(e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'Discard Driver Config', msg)
            raise e

    @loggingInfo
    def signDriverConfiguration(self, icepap_name, driver_addr, signature):
        try:
            if icepap_name in self.iPaps:
                axis = self.iPaps[icepap_name][driver_addr]
                if Mode.CONFIG != axis.mode:
                    axis.set_config()
                axis.set_config(signature)
        except RuntimeError as e:
            msg = 'Error signing config.\n{0}'.format(e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'Sign Driver', msg)
            raise e

    @loggingInfo
    def startConfiguringDriver(self, icepap_name, driver):
        try:
            mode = self.iPaps[icepap_name][driver.addr].mode
            if Mode.PROG == mode:
                return mode[0]
            if Mode.CONFIG != mode:
                self.iPaps[icepap_name][driver.addr].set_config()
            driver.setMode(Mode.CONFIG)
        except RuntimeError as e:
            msg = 'Error starting config.\n{0}'.format(e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'config', msg)
            raise e
        return mode[0]

    @loggingInfo
    def endConfiguringDriver(self, icepap_name, driver):
        try:
            if icepap_name not in self.iPaps:
                return
            axis = self.iPaps[icepap_name][driver.addr]
            if Mode.PROG == axis.mode:
                return
            if Mode.OPER != axis.mode:
                last_signature = axis.config
                axis.set_config(last_signature)
            driver.setMode(Mode.OPER)
        except RuntimeError as e:
            msg = 'Error ending config for driver ' \
                  '{0}.\n{1}'.format(driver.addr, e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'End Config', msg)
            raise e

    @loggingInfo
    def getDriverStatus(self, icepap_name, driver_addr):
        """
        Returns an array with the Status, Limit Switches and Current of the
        driver
        """
        if not self.iPaps[icepap_name].connected:
            state = (-1, False, -1)
            return state
        if self.programming_ipap is not None:
            state = (-1, False, -1)
            return state
        try:
            axis_state = self.iPaps[icepap_name].get_states(driver_addr)[0]
            # TODO: Change the ui to use boolean instead of integers
            power = int(axis_state.is_poweron())
            cfg_current = self.iPaps[icepap_name][driver_addr].get_cfg('NCURR')
            current = cfg_current['NCURR']
            status_register = axis_state.status_register
            return status_register, power, current
        except Exception as e:
            self.log.error('Failed to retrieve status for driver %d: %s',
                           driver_addr, e)
            return -1, False, -1

    @loggingInfo
    def getDriverTestStatus(self, icepap_name, driver_addr, pos_sel, enc_sel):
        """
        Returns an array with the Status, Limit Switches and Position of
        the driver
        """
        if self.programming_ipap is not None:
            return -1, -1, [-1, -1]

        axis_state = self.iPaps[icepap_name].get_states(driver_addr)[0]
        register = axis_state.status_register
        power = int(axis_state.is_poweron())

        axis = self.iPaps[icepap_name][driver_addr]
        try:
            position = axis.get_pos(pos_sel)
        except Exception as e:
            self.log.error('Failed to retrieve position for driver %d: %s',
                           driver_addr, e)

            position = -1

        try:
            encoder = axis.get_enc(enc_sel)
        except Exception as e:
            self.log.error('Failed to retrieve encoder for driver %d: %s',
                           driver_addr, e)
            encoder = -1

        return register, power, [position, encoder]

    @loggingInfo
    def getDriverActiveStatus(self, icepap_name, driver_addr):
        try:
            active = self.iPaps[icepap_name][driver_addr].active
            if active:
                return 'YES'
            else:
                return 'NO'
        except Exception as e:
            msg = 'Failed to read activation status for ' \
                  'driver {0}.\n{1}'.format(driver_addr, e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'Get Active', msg)
            raise e

    @loggingInfo
    def readIcepapParameters(self, icepap_name, driver_addr, par_list):
        axis = self.iPaps[icepap_name][driver_addr]
        try:
            values = []
            for name in par_list:
                if type(name) == type(values):
                    cmd = '?{0} {1}'.format(name[0], name[1])
                else:
                    cmd = '?{0}'.format(name)

                value = ' '.join(axis.send_cmd(cmd))
                values.append([name, value])

            return values
        except Exception as e:
            msg = 'Failed to read parameter {0} ' \
                  'for driver {1}.\n{2}'.format(name, driver_addr, e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'Read Param', msg)
            raise e

    @loggingInfo
    def writeIcepapParameters(self, icepap_name, driver_addr, par_var_list):
        # This method is only called with the command widgets changes, not with
        # the configuration widgets changes. For that reason it is not
        # needed to send the parameter in any specific order.
        axis = self.iPaps[icepap_name][driver_addr]
        for param, value in par_var_list:
            cmd = '{0} {1}'.format(param, value)
            try:
                axis.send_cmd(cmd)
            except Exception as e:
                msg = 'Failed to write icepap ' \
                      'parameter {0} = {1}\n{2}'.format(param, value, e)
                self.log.error(msg)
                MessageDialogs.showErrorMessage(None, 'Write Param', msg)
                raise e

    @loggingInfo
    def configDriverToDefaults(self, icepap_name, driver_addr):
        try:
            self.iPaps[icepap_name][driver_addr].set_cfg('DEFAULT')
        except RuntimeError as e:
            msg = 'Error configuring driver {0} to ' \
                  'defaults.\n{1}'.format(driver_addr, e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'Default Config', msg)
            raise e

    @loggingInfo
    def getDriverMotionValues(self, icepap_name, driver_addr):
        try:
            speed = self.iPaps[icepap_name][driver_addr].velocity
            acc = self.iPaps[icepap_name][driver_addr].acctime
        except RuntimeError as e:
            msg = 'Failed to retrieve motion values(velocity, acctime) ' \
                  'from driver {0}.\n{1}'.format(driver_addr, e)
            self.log.error(msg)
            MessageDialogs.showErrorMessage(None, 'Driver Motion Values', msg)
            raise e
        return speed, acc

    @loggingInfo
    def setDriverMotionValues(self, icepap_name, driver_addr, values):
        try:
            self.iPaps[icepap_name][driver_addr].velocity = values[0]
            self.iPaps[icepap_name][driver_addr].acctime = values[1]
            return 0
        except Exception as e:
            msg = 'Failed to set motion values for ' \
                  'driver {0}.\n{1}'.format(driver_addr, e)
            self.log.error(msg)
            return -1

    @loggingInfo
    def setDriverPosition(self, icepap_name, driver_addr, pos_sel, position):
        try:
            self.iPaps[icepap_name][driver_addr].set_pos(pos_sel, position)
            return 0
        except Exception as e:
            msg = 'Failed to set driver position for ' \
                  'selector {0}\n{1}'.format(pos_sel, e)
            self.log.error(msg)
            return -1

    @loggingInfo
    def setDriverEncoder(self, icepap_name, driver_addr, enc_sel, position):
        try:
            self.iPaps[icepap_name][driver_addr].set_enc(enc_sel, position)
            return 0
        except Exception as e:

            self.log.error('Failed to set encoder position for selector %d: '
                           '%s', enc_sel, e)
            return -1

    @catchError()
    def moveDriver(self, icepap_name, driver_addr, steps):
        axis = self.iPaps[icepap_name][driver_addr]
        if Mode.CONFIG == axis.mode:
            # CMOVE ONLY ALLOWS ABSOLUTE POSITIONS, IT SHOULD BE CALCULATED
            # self.iPaps[icepap_name].cmove(driver_addr, steps)
            new_pos = axis.pos + int(steps)
            axis.cmove(new_pos)
        else:
            axis.rmove(steps)

    @catchError()
    def moveDriverAbsolute(self, icepap_name, driver_addr, pos):
        axis = self.iPaps[icepap_name][driver_addr]
        if Mode.CONFIG == axis.mode:
            axis.cmove(pos)
        else:
            axis.move(pos)

    @catchError()
    def stopDriver(self, icepap_name, driver_addr):
        self.iPaps[icepap_name][driver_addr].stop()

    @catchError()
    def abortDriver(self, icepap_name, driver_addr):
        self.iPaps[icepap_name][driver_addr].abort()

    @catchError()
    def blinkDriver(self, icepap_name, driver_addr, secs):
        self.iPaps[icepap_name][driver_addr].blink(secs)

    @catchError()
    def jogDriver(self, icepap_name, driver_addr, speed):
        axis = self.iPaps[icepap_name][driver_addr]
        if Mode.CONFIG != axis.mode:
            self.iPaps[icepap_name][driver_addr].jog(speed)
        else:
            self.iPaps[icepap_name][driver_addr].cjog(speed)

    @catchError('Check driver status')
    def enableDriver(self, icepap_name, driver_addr):
        self.iPaps[icepap_name][driver_addr].power = True

    @catchError('Check driver status')
    def disableDriver(self, icepap_name, driver_addr):
        self.iPaps[icepap_name][driver_addr].power = False

    @catchError()
    def checkIcepapStatus(self, icepap_name):
        if icepap_name not in self.iPaps \
                or not self.iPaps[icepap_name].connected:
            return False
        return True

    @loggingInfo
    def isExpertFlagSet(self, icepap_name, driver_addr):
        axis = self.iPaps[icepap_name][driver_addr]
        try:
            return axis.get_cfg('EXPERT')['EXPERT']
        except Exception:
            return 'NO'

    @loggingInfo
    def _parseDriverTemplateFile(self):
        self.config_parameters = []
        doc = minidom.parse(self.config_template)
        root = doc.documentElement
        for section in root.getElementsByTagName("section"):
            section_name = ''
            if section.nodeType == Node.ELEMENT_NODE:
                section_name = section.attributes.get('name').value
            inTestSection = (section_name == "test")
            if not inTestSection:
                for pars in section.getElementsByTagName("par"):
                    if pars.nodeType == Node.ELEMENT_NODE:
                        parname = pars.attributes.get('name').value
                        self.config_parameters.append(str(parname))

    @loggingInfo
    def upgradeDrivers(self, icepap_name, progress_dialog):
        if self.programming_ipap is not None:
            return False
        self.programming_ipap = self.iPaps[icepap_name]
        self.progress_dialog = progress_dialog
        self.progress_dialog.show()
        # TODO: Remove on the future QT depency
        self.updateProgressBarTimer = Qt.QTimer()
        self.updateProgressBarTimer.timeout.connect(self.updateProgressBar)
        try:
            self.programming_ipap.mode = 'PROG'
        except Exception as e:
            print("icepapcontroller:upgradeDrivers:Some error trying to set "
                  "mode PROG:", str(e))
            return False
        self.programming_ipap.prog('DRIVERS')
        self.updateProgressBarTimer.start(2000)
        return True

    def updateProgressBar(self):
        try:
            status = self.programming_ipap.get_prog_status()
        except Exception:
            return

        value = status[0].upper()
        if value == 'DONE':
            self.progress_dialog.setValue(100)
            self.updateProgressBarTimer.stop()
            self.programming_ipap.mode = 'OPER'
            self.programming_ipap = None
        else:
            value = float(value)
            self.progress_dialog.setValue(value)

    def _wait_programming(self, ipap, logger):
        # wait process to finish
        wait = True
        while wait:
            try:
                if not ipap.connected:
                    raise RuntimeError('No connection available')
                status = ipap.get_prog_status()
                if status[0].upper() != 'DONE':
                    logger.put('Programming {}%'.format(status))
                    time.sleep(5)
                else:
                    wait = False
            except RuntimeError:
                logger.put('Connection lost, waiting for reconnection...')
                while not ipap.connected:
                    time.sleep(0.5)
                logger.put('Reconnected!')

    def _get_ipap_and_ver(self, host, port, logger):

        logger.put("Connecting to: {} {}".format(host, port))

        ipap = IcePAPController(host, port)

        try:
            curr_ver = ipap.fver
        except Exception as e:
            logger.put('Can not read the current version. {}'.format(e))
            curr_ver = -1
        logger.put('Current firmware version is {}'.format(curr_ver))

        logger.put("Setting MODE PROG")
        ipap.mode = 'PROG'
        logger.put('Mode {}'.format(ipap.mode))
        logger.put('Programing...')

        return ipap, curr_ver

    def _end_programming(self, ipap, logger):
        host = ipap._comm.host
        port = ipap._comm.port

        logger.put('Waiting 5 sec to change Mode to OPER')
        time.sleep(5)
        logger.put("Setting MODE OPER")
        ipap.mode = 'oper'
        if ipap.mode.lower() != 'oper':
            logger.put('It was not possible to change mode to OPER')
        logger.put('Mode {}'.format(ipap.mode))
        logger.put('Reboot system')
        ipap.reboot()
        logger.put('Wait while rebooting... 1 minute')
        for i in range(4):
            time.sleep(15)
            logger.put('Passed {} seconds'.format(15*(i+1)))

        try:
            ipap = IcePAPController(host, port)
        except Exception:
           logger.put('ERROR: Can not connect to the icepap after to restart')
           return

        logger.put('Connected after to restart')
        try:
            curr_ver = ipap.fver
        except Exception as e:
            logger.put('Can not read the current version. {}'.format(e))
            logger.put('CHECK Icepap!!!')
            return
        logger.put('Current version: {}'.format(curr_ver))
        logger.put('\n\nProgramming sequence done!')

    def upgradeAutomaticFirmware(self, host, port, filename, log_queue):
        log_queue.put('Upgrade Automatic Firmware')

        ipap, curr_ver = self._get_ipap_and_ver(host, port, log_queue)
        if filename != '':
            log_queue.put('Saving new firmware on the icepap')
            ipap.sprog(filename, saving=True)
        else:
            log_queue.put('Using saved firmware on the icepap')
            try:
                ver_saved = ipap.ver_saved.system[0]
                log_queue.put('Version saved: {}'.format(ver_saved))
            except Exception:
                log_queue.put('Can not read version saved')


        log_queue.put('Send PROG ALL')
        ipap.prog('ALL', force=True)
        self._wait_programming(ipap, log_queue)

        if curr_ver < 2:
            ipap.prog('MCPU0')
            ipap.prog('MCPU1')
            ipap.prog('MCPU2')

        time.sleep(5)
        self._end_programming(ipap, log_queue)
        if curr_ver < 3.21:
            try:
                ipap = IcePAPController(host, port)
                new_ver = ipap.fver
                if new_ver >= 3.21:
                    log_queue.put('Restarting first time absolut encoder '
                                    'registers')
                    ipap.send_cmd(':ISG ABSRST')
            except Exception:
                log_queue.put('ERROR!!!!\nCan not restart absolut enconder '
                                'registers')

    @loggingInfo
    def upgradeFirmware(self, host, port, filename, component, options, force,
                        log_queue):

        ipap, _ = self._get_ipap_and_ver(host, port, log_queue)

        if filename != '':
            save = False
            if options == 'SAVE':
                save = True
                options = ''
            ipap.sprog(filename, component, force, save, options)
        else:
            ipap.prog(component, force)

        self._wait_programming(ipap, log_queue)
        self._end_programming(ipap, log_queue)

    @loggingInfo
    def testConnection(self, host, port):
        try:
            ipap = IcePAPController(host, port)
            if self.log.isEnabledFor(logging.INFO):
                self.log.info('Testing connection to: %s:%s version: %s',
                              host, port, ipap.fver)
            ipap.disconnect()
            return True
        except Exception:
            return False

    @loggingInfo
    def find_networks(self, configs, addr_pattern, mask_pattern):
        addr_parse = re.compile(addr_pattern)
        mask_parse = re.compile(mask_pattern)
        networks = []
        for config in configs:
            addr = addr_parse.search(config)
            mask = mask_parse.search(config)
            if addr and mask:
                net = IP(addr.group(2)+"/"+mask.group(2), make_net=True)
                networks.append(net)
        return networks

    @loggingInfo
    def host_in_same_subnet(self, host):
        if self._config._options.allnets or \
                self._config.config['all_networks']['use']:
            return True

        networks = []
        configs = None
        addr_pattern = None
        mask_pattern = None
        digits = r'[0-9]{1,3}'

        if os.name == 'posix':
            ifconfigs = ['/sbin/ifconfig', '/usr/sbin/ifconfig',
                         '/bin/ifconfig', '/usr/bin/ifconfig']
            ifconfig = list(filter(os.path.exists, ifconfigs))[0]
            fp = os.popen(ifconfig+' -a')
            configs = fp.read().split('\n\n')
            fp.close()
            # 180214 - Adapted to Debian9 "/sbin/ifconfig -a" output
            # addr_pattern = r'(inet addr:) *(%s\.%s\.%s\.%s)[^0-9]' \
            #   % ((digits,)*4)
            # mask_pattern = r'(Mask:) *(%s\.%s\.%s\.%s)[^0-9]' \
            #   % ((digits,)*4)
            addr_pattern = r'(inet ) *(%s\.%s\.%s\.%s)' \
                           r'[^0-9]' % ((digits,)*4)
            mask_pattern = r'(netmask ) *(%s\.%s\.%s\.%s)' \
                           r'[^0-9]' % ((digits,)*4)
        elif os.name == 'nt':
            fp = os.popen('ipconfig /all')
            configs = fp.read().split(':\r\n\r\n')
            fp.close()
            # 180214 - Adapted to Windows7 "ipconfig /all" output
            # addr_pattern = r'(IP Address).*: (%s\.%s\.%s\.%s)[^0-9]' % ((
            # digits,)*4)
            addr_pattern = r'(IPv4 Address).*: (%s\.%s\.%s\.%s)' \
                           r'[^0-9]' % ((digits,)*4)
            mask_pattern = r'(Subnet Mask).*: (%s\.%s\.%s\.%s)' \
                           r'[^0-9]' % ((digits,)*4)

        if configs and addr_pattern and mask_pattern:
            networks = self.find_networks(configs, addr_pattern, mask_pattern)

        if len(networks) > 0:
            host_addr = socket.gethostbyname(host)
            for net in networks:
                if host_addr in net:
                    return True
            return False
        else:
            msg = "Sorry system not yet supported.\nWe allow you access to " \
                  "the icepap even if it is in another subnet."
            MessageDialogs.showInformationMessage(None,
                                                  "Not posix operating system",
                                                  msg)
            return True



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtCore, QtWidgets, uic
from pkg_resources import resource_filename
import logging
from ..lib import MainManager
from ..helpers import loggingInfo


class DialogAddIcepap(QtWidgets.QDialog):
    log = logging.getLogger('{}.DialogAddIcepap'.format(__name__))

    @loggingInfo
    def __init__(self, parent, location, test_mode=False):
        QtWidgets.QDialog.__init__(self, parent)
        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'dialogaddicepap.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)

        self.modal = True
        self.ui.txtPort.setText("5000")
        if test_mode:
            return
        self.buildLocationCombo(location)

    @loggingInfo
    def buildLocationCombo(self, location):
        for location_name in list(MainManager().locationList.keys()):
            self.ui.cbLocation.addItem(location_name)
        self.ui.cbLocation.setCurrentIndex(
            self.ui.cbLocation.findText(location, QtCore.Qt.MatchFixedString))

    @loggingInfo
    def getData(self):
        host = str(self.ui.txtHost.text())
        port = str(self.ui.txtPort.text())
        desc = str(self.ui.txtDescription.toPlainText())
        location = str(self.ui.cbLocation.currentText())
        return [host, port, desc, location]

    @loggingInfo
    def setData(self, name, host, port, description, location):
        self.ui.txtHost.setEnabled(False)
        self.ui.txtPort.setEnabled(False)
        self.ui.txtHost.setText(host)
        self.ui.txtPort.setText(str(port))
        self.ui.txtDescription.insertPlainText(description)
        self.ui.cbLocation.setCurrentIndex(
            self.ui.cbLocation.findText(location, QtCore.Qt.MatchFixedString))


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)

    w = DialogAddIcepap(None, 'test', True)
    w.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtWidgets, QtGui, Qt, uic
from pkg_resources import resource_filename
from xml.dom import minidom, Node
import logging
from .messagedialogs import MessageDialogs
from ..lib import MainManager, StormManager
from ..helpers import loggingInfo


class DialogDriverConflict(QtWidgets.QDialog):
    log = logging.getLogger('{}.DialogDriverConflict'.format(__name__))

    @loggingInfo
    def __init__(self, parent, driver, test_mode=False):
        QtWidgets.QDialog.__init__(self, parent)

        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'dialogdriverconflict.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)
        self.modal = True
        font = QtGui.QFont()
        font.setPointSize(7.5)
        self.setFont(font)
        self.config_template = resource_filename('icepapcms.templates',
                                                 'driverparameters.xml')
        if test_mode:
            return
        self._driver = driver
        self._manager = MainManager()
        self.ui.tableWidget.horizontalHeader().setSectionResizeMode(
            0, Qt.QHeaderView.Stretch)
        self.ui.tableWidget.horizontalHeader().setSectionResizeMode(
            1, Qt.QHeaderView.Stretch)
        self.ui.tableWidget.horizontalHeader().setSectionResizeMode(
            2, Qt.QHeaderView.Stretch)
        self._fillTable()

        # Connect Signals
        self.ui.btnStored.clicked.connect(self.btnStored_on_click)
        self.ui.btnIcepap.clicked.connect(self.btnIcepap_on_click)

    @loggingInfo
    def _createTableView(self):
        self.var_dict = {}
        doc = minidom.parse(self.config_template)
        root = doc.documentElement
        row = 0
        for section in root.getElementsByTagName("section"):
            if section.nodeType == Node.ELEMENT_NODE:
                section_name = section.attributes.get('name').value
            inTestSection = (section_name == "test")
            if inTestSection:
                return
            for pars in section.getElementsByTagName("par"):
                if pars.nodeType == Node.ELEMENT_NODE:
                    self.ui.tableWidget.insertRow(row)
                    parname = pars.attributes.get('name').value
                    parname = parname.strip()
                    self.var_dict[parname] = row
                    self._addItemToTable(row, 0, parname, False)
                    row = row + 1

    @loggingInfo
    def _fillTable(self):
        self._createTableView()
        self.icepap_values = self._manager.getDriverConfiguration(
            self._driver.icepapsystem_name, self._driver.addr)
        row = 0
        current = False

        for row in range(self.ui.tableWidget.rowCount()):
            name = str(self.ui.tableWidget.item(row, 0).text())

            if name == "NAME":
                name = str("IPAPNAME")

            stored_value = self._driver.current_cfg.getParameter(name, True)
            if stored_value is None:
                stored_value = ""

            icepap_value = self.icepap_values.getParameter(name, True)
            if icepap_value is None:
                icepap_value = ""
            color = stored_value != icepap_value

            self._addItemToTable(row, 1, stored_value, color)
            self._addItemToTable(row, 2, icepap_value, color)

        if current:
            MessageDialogs.showWarningMessage(
                self, "Driver conflict",
                "Warning!!\nCurrent values (IN, IB, II) have changed!!\n")

    @loggingInfo
    def btnIcepap_on_click(self):
        db = StormManager()
        db.store(self.icepap_values)
        # bug sqlite
        self._driver.addConfiguration(self.icepap_values)
        self.accept()

    @loggingInfo
    def btnStored_on_click(self):
        del self.icepap_values
        self._manager.saveValuesInIcepap(self._driver,
                                         self._driver.current_cfg.toList())
        self.accept()

    @loggingInfo
    def _addItemToTable(self, row, column, text, colored):
        item = QtWidgets.QTableWidgetItem()
        item.setText(text)
        item.setFlags(Qt.Qt.ItemIsSelectable)
        if colored:
            item.setTextColor(QtGui.QColor(Qt.Qt.red))
        self.ui.tableWidget.setItem(row, column, item)


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)

    w = DialogDriverConflict(None, None, True)
    w.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from PyQt5 import QtWidgets, uic
from pkg_resources import resource_filename
import ldap3
import logging
import re


class DialogLdapLogin(QtWidgets.QDialog):
    log = logging.getLogger('{}.DialogLdapLogin'.format(__name__))

    def __init__(self, parent=None, config=None):
        """
        Widget to verify the user on LDAP servers
        :param parent: QtWidget
        :param config: LDAP configuration dictionary it must have three keys:
                       - not_allowed: <list> list of users not allowed to use
                                      the application e.g: [root]
                       - servers: <list> list of servers to validate the users
                                  e.g: [ldap.server.com]
                       - user_template: <str> string format to login on the
                                        server e.g: 'uid={},ou=People'
        """
        QtWidgets.QDialog.__init__(self, parent)
        ui_filename = resource_filename('icepapcms.gui.ldap.ui', 'login.ui')
        self.ui = self
        self.username = None
        uic.loadUi(ui_filename, baseinstance=self.ui)
        self.modal = True
        self.flag_error = False

        try:
            self.re_not_allowed = config['not_allowed']
            self.ldap_servers = config['servers']
            self.ldap_user_template = config['user_template']
            self.regex = '(?:% s)' % '|'.join(self.re_not_allowed)
        except Exception as e:
            self.log.error('Wrong ldap configuration see docstring. Error %s',
                           str(e))
            self.flag_error = True

        self.dialog_buttons.accepted.connect(self.ok_clicked)
        self.dialog_buttons.rejected.connect(self.reject)
        if not self.flag_error:
            self.user.textChanged.connect(self.user_change)

    def user_change(self, text):
        if re.match(self.regex, text):
            self.user.setStyleSheet('color: red;')
        else:
            self.user.setStyleSheet('color: black;')

    def validate(self):
        if self.flag_error:
            QtWidgets.QMessageBox.critical(
                self, 'Login Error',
                'Wrong ldap configuration contact with support')
            self.clear()
            return False

        user = self.user.text()
        if re.match(self.regex, user):
            self.log.error('Try to log as %s', user)
            QtWidgets.QMessageBox.critical(self, "Login Error",
                                           "User not allowed")
            self.clear()
            return False

        passwd = self.password.text()

        for server in self.ldap_servers:
            s = ldap3.Server(server)
            ldap_user = self.ldap_user_template.format(user)
            connection = ldap3.Connection(s, ldap_user, passwd)
            if connection.bind():
                self.log.info('Login as %s', user)
                self.username = user
                return True

        QtWidgets.QMessageBox.critical(
            self, "Login Error", "Your username or password is incorrect")
        self.clear()
        return False

    def clear(self):
        self.password.setText('')
        self.user.setText('')
        self.user.setFocus()

    def ok_clicked(self):
        if self.validate():
            self.accept()


def main():
    logging.basicConfig(level=logging.INFO)
    import sys
    app = QtWidgets.QApplication(sys.argv)
    d = QtWidgets.QWidget()
    w = DialogLdapLogin(d)
    w.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtWidgets, uic
from pkg_resources import resource_filename
import os
import time
import logging
import threading
from ..lib.sanpshot import IcepapSnapshot
from ..lib.configmanager import ConfigManager
from .messagedialogs import MessageDialogs


class DialogSnapshot(QtWidgets.QDialog):

    def __init__(self, parent, host='', port=5000):
        QtWidgets.QDialog.__init__(self, parent)

        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'snapshot.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)
        self.modal = True
        self._config = ConfigManager()
        self.ui.host.setText(host)
        self.ui.port.setText(str(port))
        self.set_filename()

        self.ui.browser.clicked.connect(self.set_filename)
        self.ui.take_snapshot.clicked.connect(self.take_snapshot_clicked)

    def set_filename(self):
        if self.ui.filename.text() == '':
            name = 'snapshot_{}.yaml'.format(time.strftime('%Y%m%d_%H%M%S'))
            snapshots_folder = \
                self._config.config[self._config.icepap]['snapshots_folder']
            directory = os.path.join(snapshots_folder, name)
        else:
            directory = self.ui.filename.text()
        fn = QtWidgets.QFileDialog.getSaveFileName(
            self, "Take Snapshot",
            directory=directory,
            filter='*.yaml')[0]
        self.ui.filename.setText(fn)

    def update_progress_bar(self, ipap_snapshot):
        while ipap_snapshot.done != 100:
            self.ui.progress_bar.setValue(ipap_snapshot.done)
            time.sleep(1)
        self.ui.progress_bar.setValue(ipap_snapshot.done)

    def take_snapshot_clicked(self):
        try:
            host = self.ui.host.text()
            if host == '':
                MessageDialogs.showErrorMessage(
                    self, 'Error', 'You should introduce a valid host')
            port = int(self.ui.port.text())
            filename = self.ui.filename.text()
            try:
                with open(filename, 'w'):
                    pass
            except Exception as e:
                MessageDialogs.showErrorMessage(
                    self, 'Error on file creation', str(e))
                return

            try:
                ipap_snapshot = IcepapSnapshot(host, port)
            except Exception as e:
                MessageDialogs.showErrorMessage(self, 'Error connecting',
                                                str(e))
                return
            thread = threading.Thread(target=self.update_progress_bar,
                                      args=[ipap_snapshot])

            self.ui.take_snapshot.setEnabled(False)
            thread.start()
            ipap_snapshot.create_snapshot(filename)
            thread.join()
        finally:
            self.ui.take_snapshot.setEnabled(True)


def main():
    import sys
    app = QtWidgets.QApplication(sys.argv)

    w = DialogSnapshot(None)
    w.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtWidgets, uic, Qt
from pkg_resources import resource_filename
import sys
import datetime
import logging
from icepap import IcePAPController
import threading
import queue
from .messagedialogs import MessageDialogs
from ..lib import IcepapsManager, ConfigManager
from ..helpers import loggingInfo


class thread_with_trace(threading.Thread):
    def __init__(self, *args, **keywords):
        threading.Thread.__init__(self, *args, **keywords)
        self.killed = False

    def start(self):
        self.__run_backup = self.run
        self.run = self.__run
        threading.Thread.start(self)

    def __run(self):
        sys.settrace(self.globaltrace)
        self.__run_backup()
        self.run = self.__run_backup

    def globaltrace(self, frame, event, arg):
        if event == 'call':
            return self.localtrace
        else:
            return None

    def localtrace(self, frame, event, arg):
        if self.killed:
            if event == 'line':
                raise SystemExit()
        return self.localtrace

    def kill(self):
        self.killed = True


class DialogIcepapProgram(QtWidgets.QDialog):
    log = logging.getLogger('{}.DialogIcepapProgram'.format(__name__))

    @loggingInfo
    def __init__(self, parent, test_mode=False):
        QtWidgets.QDialog.__init__(self, parent)
        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'dialogipapprogram.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)

        self.timer = Qt.QTimer(self)
        self.ui.btnAbort.setStyleSheet('text-align:left;')

        # Signals
        self.ui.btnBrowser.clicked.connect(self.btnBrowse_on_click)
        self.ui.btnClose.clicked.connect(self.close)
        self.ui.btnTest.clicked.connect(self.btnTest_on_click)
        self.ui.btnProgram.clicked.connect(self.btnProgram_on_click)
        self.ui.cbProgram.currentTextChanged.connect(self.cbProgram_changed)
        self.ui.btnAbort.clicked.connect(self.abort_clicked)
        self.ui.btnSnapshot.clicked.connect(self.take_snapshot)
        self.timer.timeout.connect(self._check_queue)

        self.ui.sbAddr.setDisabled(True)
         # BY DEFAULT, PROGRAMM ALL INSTEAD OF NONE
        self.ui.cbProgram.setCurrentIndex(1)
        self.ui.expert.setChecked(False)
        if test_mode:
            return
        self._ipapctrl = IcepapsManager()
        self.log_queue = queue.Queue()
        self.programming = False
        self._btn_enables(True)
        self._thread = None
        self.flg_abort = False

    @loggingInfo
    def btnBrowse_on_click(self):
        folder = ConfigManager().config["icepap"]["firmware_folder"]
        fn = QtWidgets.QFileDialog.getOpenFileName(self, "Open Firmware File",
            str(folder), str("*.*"))
        if fn[0] == '':
            return
        filename = str(fn[0])
        self.ui.txtFirmwareFile.setText(filename)

    @loggingInfo
    def cbProgram_changed(self, text):
        if text == "ADDR":
            self.ui.sbAddr.setEnabled(True)
        else:
            self.ui.sbAddr.setDisabled(True)

    def closeEvent(self, event) -> None:
        if self.programming:
            self.abort_clicked()
            if self.programming:
                event.ignore()
        else:
            self.timer.stop()

    @loggingInfo
    def btnTest_on_click(self):
        host = self.ui.txtHost.text()
        if self._ipapctrl.testConnection(host, 5000):
            self.addToLog("Connection OK")
        else:
            self.addToLog("Connection error")
            MessageDialogs.showWarningMessage(
                self, "Connection error", "Icepap not reachable.")

    @loggingInfo
    def btnProgram_on_click(self):
        try:
            self.programming = True
            self._btn_enables(False)
            self.timer.start(2000)
            self._thread = thread_with_trace(target=self.startProgramming)
            self._thread.daemon = True
            self._thread.start()
        except Exception as e:
            self.addToLog(str(e))

    def _btn_enables(self, enabled):
        self.ui.btnProgram.setEnabled(enabled)
        self.ui.btnSnapshot.setEnabled(enabled)
        self.ui.btnTest.setEnabled(enabled)
        self.ui.btnBrowser.setEnabled(enabled)
        self.ui.txtHost.setEnabled(enabled)
        self.ui.txtFirmwareFile.setEnabled(enabled)
        self.ui.expert.setEnabled(enabled)
        self.ui.btnAbort.setEnabled(not enabled)

    def _check_queue(self):
        while not self.log_queue.empty():
            try:
                value = self.log_queue.get()
                self.addToLog(value)
            except queue.Empty:
                pass

        if not self.programming:
            while not self.log_queue.empty():
                try:
                    value = self.log_queue.get()
                    self.addToLog(value)
                except queue.Empty:
                    pass
            self.timer.stop()
            self._btn_enables(True)
            self._thread = None

    def take_snapshot(self):
        self.parent().actionSnapshot.trigger()

    def abort_clicked(self):
        if self._thread is None or not self.programming:
            return

        msg = Qt.QMessageBox(self)
        msg.setIcon(Qt.QMessageBox.Question)
        msg.setText('Are you sure to abort the programming?\n'
                    'It can leave the icepap in bad condition')
        msg.setWindowTitle("Warning")
        msg.addButton(Qt.QMessageBox.Yes)
        msg.addButton(Qt.QMessageBox.No)
        result = msg.exec_()
        if result == Qt.QMessageBox.Yes:
            self._thread.kill()
            self._thread.join()
            self.programming = False

    @loggingInfo
    def startProgramming(self):
        try:
            filename = self.ui.txtFirmwareFile.text()
            host = self.ui.txtHost.text()
            if host == '':
                MessageDialogs.showErrorMessage(
                    self, 'Empty hostname', 'You must enter a valid hostname')
                return
            port = 5000
            if self.ui.expert.isChecked():
                component = self.ui.cbProgram.currentText()
                if component == "ADDR":
                    addr = self.ui.sbAddr.value()
                    if addr not in IcePAPController.ALL_AXES_VALID:
                        MessageDialogs.showErrorMessage(self, 'Wrong Axis',
                                                        'Axis value not valid')
                    component = str(addr)

                options = self.ui.cbOptions.currentText()
                force = False
                if self.ui.chkForce.isChecked():
                    force = True
                self._ipapctrl.upgradeFirmware(host, port, filename, component,
                                               options, force, self.log_queue)
            else:
                self._ipapctrl.upgradeAutomaticFirmware(host, port, filename,
                                                        self.log_queue)
        except Exception as e:
            MessageDialogs.showErrorMessage(None, 'Error programming', str(e))
        finally:
            self.programming = False

    @loggingInfo
    def addToLog(self, text):
        t = datetime.datetime.now().strftime("%H:%M:%S")
        self.ui.txtLog.append(t + "> " + text)
        self.ui.txtLog.ensureCursorVisible()


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)

    w = DialogIcepapProgram(None, True)
    w.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtGui, Qt, QtWidgets, uic, QtCore
from pkg_resources import resource_filename
from icepap import Mode, State
import logging
from .Led import Led
from ..lib import MainManager, Conflict
from ..helpers import loggingInfo


class IcePapDriverWidget(QtWidgets.QWidget):
    icepapDoubleClicked = QtCore.pyqtSignal(object)
    log = logging.getLogger('{}.IcePapDriverWidget'.format(__name__))

    @loggingInfo
    def __init__(self, parent=None, BigSize=True, test_mode=False):
        QtWidgets.QWidget.__init__(self, parent)
        self.initView(BigSize, test_mode)
        self.MaxCurrent = 7
        self._driver = None
        self.setMouseTracking(True)
        self.status = -1
        self.ready = -1
        self.power = -1
        self.mode = -1

    @loggingInfo
    def initView(self, Big, test_mode=False):
        self.BigSize = Big
        self.coloroff = QtGui.QColor(225, 255, 200)
        self.colorerror = QtGui.QColor(255, 179, 179)
        self.colorok = QtGui.QColor(223, 223, 237)
        self.colorwarning = QtGui.QColor(255, 255, 0)
        self.colorconfig = QtGui.QColor(255, 206, 162)
        self.ui = QtWidgets.QWidget(self)

        if Big:
            ui_filename = resource_filename('icepapcms.gui.ui',
                                            'icepapdriverwidget.ui')
        else:
            ui_filename = resource_filename('icepapcms.gui.ui',
                                            'icepapdriverwidgetsmall.ui')

        uic.loadUi(ui_filename, baseinstance=self.ui,
                   package='icepapcms.gui.Led')
        if Big:
            self.setPaletteColor(self.ui.lcdCurrent, self.coloroff,
                                 QtGui.QColor(Qt.Qt.white))

        self.ui.ledStatus.changeColor(Led.YELLOW)
        self.ui.ledLimitPos.changeColor(Led.ORANGE)
        self.ui.ledLimitNeg.changeColor(Led.GREEN)
        if test_mode:
            return
        self._manager = MainManager()

        self.setAutoFillBackground(True)
        # Signals
        self.ui.pushButton.clicked.connect(self.btnEnDis_on_click)

    @loggingInfo
    def mouseDoubleClickEvent(self, event):
        self.icepapDoubleClicked.emit(event)
        event.accept()

    @loggingInfo
    def mousePressEvent(self, event):
        tooltip = str(self._driver.current_cfg)
        QtWidgets.QToolTip.showText(event.globalPos(), tooltip)
        event.accept()

    @loggingInfo
    def mouseMoveEvent(self, event):
        event.accept()

    @loggingInfo
    def btnEnDis_on_click(self, bool):
        if bool:
            self.ui.pushButton.setText("power OFF")
            self._manager.enableDriver(self._driver.icepapsystem_name,
                                       self._driver.addr)
        else:
            self.ui.pushButton.setText("power ON")
            self._manager.disableDriver(self._driver.icepapsystem_name,
                                        self._driver.addr)

    @loggingInfo
    def getDriver(self):
        return self._driver

    @loggingInfo
    def fillData(self, driver):
        self._driver = driver
        if driver is None:
            self.ui.lblName.setText("")
            self.ui.frame.setEnabled(False)
            return False
        else:
            return self.fillStatus()

    @loggingInfo
    def refresh(self):
        if self._driver is not None:
            return self.fillStatus()
        else:
            return True

    @loggingInfo
    def fillStatus(self):
        if self._driver.getName() is None:
            self.ui.lblName.setText("- %d -" % self._driver.addr)
        elif self._driver.getName() != "":
            self.ui.lblName.setText("%d- %s" % (self._driver.addr,
                                                self._driver.getName()))
        else:
            self.ui.lblName.setText("- %d -" % self._driver.addr)

        if self._driver.conflict == Conflict.DRIVER_NOT_PRESENT:
            self.setPaletteColor(self.ui.frame, self.colorerror, Qt.Qt.black)
        elif self._driver.conflict == Conflict.DRIVER_CHANGED:
            self.setPaletteColor(self.ui.frame, self.colorwarning, Qt.Qt.black)
        elif self._driver.mode == Mode.CONFIG:
            self.setPaletteColor(self.ui.frame, self.colorconfig, Qt.Qt.black)
        else:
            self.setPaletteColor(self.ui.frame, self.colorok, Qt.Qt.black)

        (status, power, current) = self._manager.getDriverStatus(
            self._driver.icepapsystem_name, self._driver.addr)
        if status == -1:
            self.ui.pushButton.setEnabled(False)
            self.ui.ledStatus.changeColor(Led.RED)
            self.ui.ledStatus.on()
            return

        axis_state = State(status)
        disabled = axis_state.get_disable_code()
        # TODO: use boolean instead of integers
        ready = int(axis_state.is_ready())
        mode = int(axis_state.get_mode_code())
        if self.status != disabled or self.mode != mode or \
                self.power != power or self.ready != ready:
            if disabled == 0:
                if power:
                    self.ui.ledStatus.changeColor(Led.GREEN)
                    self.ui.ledStatus.on()
                    self.ui.pushButton.setText("power OFF")
                    self.ui.pushButton.setChecked(True)
                    self.ui.pushButton.setEnabled(True)
                    self.mode = mode
                else:
                    self.ui.pushButton.setEnabled(True)
                    self.ui.pushButton.setText("power ON")
                    self.ui.pushButton.setChecked(False)
                    self.ui.ledStatus.changeColor(Led.RED)
                    self.ui.ledStatus.on()
            elif disabled == 1:
                # driver is not active disable motion and enable
                self.ui.pushButton.setEnabled(False)
                self.ui.ledStatus.changeColor(Led.RED)
                self.ui.ledStatus.on()
            else:
                self.ui.pushButton.setEnabled(True)
                self.ui.pushButton.setText("power ON")
                self.ui.pushButton.setChecked(False)
                self.ui.ledStatus.changeColor(Led.RED)
                self.ui.ledStatus.on()

        if status == -1:
            self.ui.pushButton.setEnabled(False)
            self.ui.ledStatus.changeColor(Led.RED)
            self.ui.ledStatus.on()

        self.status = disabled
        self.ready = ready
        self.power = power

        lower = axis_state.is_limit_negative()
        upper = axis_state.is_limit_positive()
        if lower:
            self.ui.ledLimitNeg.on()
        else:
            self.ui.ledLimitNeg.off()

        if upper:
            self.ui.ledLimitPos.on()
        else:
            self.ui.ledLimitPos.off()

        if self.BigSize:
            self.setCurrent(current)
        return True

    @loggingInfo
    def setCurrent(self, current):
        self.ui.lcdCurrent.display(current)
        current = float(current)
        color = QtGui.QColor()
        if current < 0 or current > 7:
            color = self.coloroff
        else:
            percentage = ((self.MaxCurrent)-current) / self.MaxCurrent

            S = 255 - abs(int(128*percentage))
            H = abs(int(180*percentage))
            V = 255 - abs(int(60*percentage))
            color.setHsv(H, S, V)
        self.setPaletteColor(self.ui.lcdCurrent, color,
                             QtGui.QColor(Qt.Qt.black))

    @loggingInfo
    def setPaletteColor(self, widget, backcolor, forecolor):
        palette = widget.palette()
        widget.setAutoFillBackground(True)
        palette.setColor(QtGui.QPalette.Base, backcolor)
        palette.setColor(QtGui.QPalette.Active, QtGui.QPalette.Text, forecolor)
        widget.setPalette(palette)
        widget.show()


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    lw = IcePapDriverWidget(BigSize=True, test_mode=True)
    lw.show()
    sys.exit(app.exec_())


#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5 import QtWidgets, uic
from pkg_resources import resource_filename
import logging
from ..helpers import loggingInfo


class DialogNewDriver(QtWidgets.QDialog):
    log = logging.getLogger('{}.DialogNewDriver'.format(__name__))

    @loggingInfo
    def __init__(self, parent, more_info, expertFlag):
        QtWidgets.QDialog.__init__(self, parent)
        ui_filename = resource_filename('icepapcms.gui.ui',
                                        'dialognewdriver.ui')
        self.ui = self
        uic.loadUi(ui_filename, baseinstance=self.ui)

        self.modal = True
        self.more_info = more_info
        self.expertFlag = expertFlag
        self.ui.btnCancel.setDefault(True)
        self.user_result = "CANCEL"

        if not expertFlag:
            self.ui.lblExpertInfo.setVisible(False)
            self.ui.btnUseDriver.setVisible(False)
        self.connectSignals()

    @loggingInfo
    def connectSignals(self):
        self.ui.btnUseDefaults.clicked.connect(self.btnUseDefaults_clicked)
        self.ui.btnUseDriver.clicked.connect(self.btnUseDriver_clicked)
        self.ui.btnCancel.clicked.connect(self.btnCancel_clicked)
        self.ui.btnMoreInfo.clicked.connect(self.btnMoreInfo_clicked)

    @loggingInfo
    def btnUseDefaults_clicked(self):
        self.user_result = "DEFAULT"
        self.accept()

    @loggingInfo
    def btnUseDriver_clicked(self):
        self.user_result = "DRIVER"
        self.accept()

    @loggingInfo
    def btnCancel_clicked(self):
        self.user_result = "CANCEL"
        self.reject()

    @loggingInfo
    def btnMoreInfo_clicked(self):
        self.more_info.show()

    @loggingInfo
    def result(self):
        return self.user_result


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    d = QtWidgets.QWidget()
    w = DialogNewDriver(None, d, expertFlag=True)
    w.show()
    sys.exit(app.exec_())



#!/usr/bin/env python

# -----------------------------------------------------------------------------
# This file is part of icepapcms https://github.com/ALBA-Synchrotron/icepapcms
#
# Copyright 2008-2018 CELLS / ALBA Synchrotron, Bellaterra, Spain
#
# Distributed under the terms of the GNU General Public License,
# either version 3 of the License, or (at your option) any later version.
# See LICENSE.txt for more info.
# -----------------------------------------------------------------------------


from PyQt5.QtWidgets import QMessageBox


class MessageDialogs:

    def showYesNoMessage(self, parent, caption, question):
        answer = QMessageBox.question(parent, caption, question,
                                      QMessageBox.Yes | QMessageBox.No,
                                      defaultButton=QMessageBox.No)
        return answer == QMessageBox.Yes
    showYesNoMessage = classmethod(showYesNoMessage)

    def showWarningMessage(self, parent, caption, warning):
        QMessageBox.warning(parent, caption, warning)
    showWarningMessage = classmethod(showWarningMessage)

    def showInformationMessage(self, parent, caption, info):
        QMessageBox.information(parent, caption, info)
    showInformationMessage = classmethod(showInformationMessage)

    def showErrorMessage(self, parent, caption, error):
        QMessageBox.critical(parent, caption, error)
    showErrorMessage = classmethod(showErrorMessage)


